SystemOrganization addCategory: #'RPackage-SystemIntegration'!

!RPackage methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 4/13/2011 22:45'!
importCategory
	"import all the classes of the class categories matching the receiver name (following package info semantics)." 
	
	| cat classesInCat |
	cat := Smalltalk globals organization categories select: [:each | each beginsWith:  self name ]. 
	classesInCat := cat gather: [:c | Smalltalk globals organization listAtCategoryNamed: c].
	classesInCat do: [:each | 
						self importClass: (Smalltalk globals at: each).
						self importClass: (Smalltalk globals at: each) class]! !

!RPackage methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 7/29/2010 21:14'!
importCategory: aCat forClass: aClass 
	"import all the methods of a category as defined in the receiver." 
	
		(aClass organization listAtCategoryNamed: aCat)
			do: [:each | self addMethod: (aClass >> each)]
	
	! !

!RPackage methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 5/31/2011 15:56'!
importClass: aClass 
	"import a class already created but not attached to a package to the receiver. 
	Handle also *- convention. Methods defined in *category are not added to the package.
	Pay attention that it will not import anything from the metaClass side
	"
	
	| protocols |
	self addClassDefinition: aClass.
	protocols := aClass organization categories 
		reject: [:each | (each first = $*) and: [ ( '*', self name asLowercase match: each asLowercase) not] ].
		"we reject all extensions except the ones that have the same name as the package - highly improbable but who knows"
	protocols do: [:each | self importProtocol: each forClass: aClass]
	
	! !

!RPackage methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 7/29/2010 21:16'!
importProtocol: aProtocol forClass: aClass 
	"import all the methods of a protocol as defined in the receiver." 
	
		(aClass organization listAtCategoryNamed: aProtocol)
			do: [:each | self addMethod: (aClass >> each)]
	
	! !

!RPackageOrganizer class methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 7/6/2010 13:39'!
isPackageDefinedForClass: aClass

	^ (RPackage organizer packageOf: aClass) notNil! !

!RPackageOrganizer class methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 7/6/2010 13:41'!
methodAdded: anEvent
	"precondition: package exist, class exist"
	
	"| cm |
	cm := anEvent item.
	cm isValid ifFalse: [^self]. " "to eliminate comment and doit crap"
	| methodCategory |
	methodCategory := anEvent itemProtocol.
	(methodCategory isEmptyOrNil or:[methodCategory first ~= $*]) 
		ifTrue: [ ]
		ifFalse: [
					(self isPackageDefinedForClass: anEvent itemClass)
						ifTrue: []
						ifFalse: [RPackage new named: (Smalltalk organization categoryOfElement: anEvent itemClass asSymbol) ]
					.
	].
	! !

!RPackageOrganizer class methodsFor: '*RPackage-SystemIntegration' stamp: 'sd 4/23/2011 23:31'!
registerInterestToSystemAnnouncement

	self default unregisterInterestToSystemAnnouncement.
	"To make sure that we do not have it twice registered"
	self default registerInterestToSystemAnnouncement! !

!RPackageOrganizer class methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/26/2010 10:23'!
unregisterInterestToSystemAnnouncement

	self default unregisterInterestToSystemAnnouncement! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 10/16/2011 20:57'!
announcer

	^ SystemAnnouncer privateAnnouncer! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 4/13/2011 20:18'!
packageExactlyMatchingExtensionName: anExtensionName
	"only look for a package for which the name match 'anExtensionName', making no difference about case. Return nil if no package is found"
	
	|tmpPackage tmpPackageName|
	packages keys do: [:aSymbol |
		(aSymbol sameAs: anExtensionName asSymbol)
			ifTrue: [ tmpPackage := self packageNamed: aSymbol ]].
	^ tmpPackage! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 5/13/2011 13:46'!
packageForProtocol: aProtocolName inClass: aClass
	"According aProtocolName is an extension protocol, will look for a matching package. If no matching package is found ,  return nil. If aProtocolName is not an extension protocol, return the parent package of aClass"
	
	|tmpProtocol|
	tmpProtocol := aProtocolName ifNil: [''].
	^ (tmpProtocol beginsWith: '*')
		ifTrue: [  (self packageMatchingExtensionName:  (tmpProtocol copyWithout: $*)) ]
		ifFalse: [ aClass package ]! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/25/2010 13:40'!
packageMatchingExtensionName: anExtensionName

	"return nil if no package i found"
	|tmpPackage tmpPackageName|
	"we first look if their is a package matching exactly the name specified"
	packages keys do: [:aSymbol |
		(aSymbol sameAs: anExtensionName asSymbol)
			ifTrue: [^ self packageNamed: aSymbol]
		].
	
	"if no package was found, we try to find one matching the begining of the name specified"
	tmpPackageName := ''.
	packages keys do: [:aSymbol |
		(aSymbol asString,'*'  match: anExtensionName)
			ifTrue: [
				"we keep the longest package name found"
				(aSymbol size > tmpPackageName size) ifTrue: [
					tmpPackageName := aSymbol.
					tmpPackage := self packageNamed: aSymbol
					].
				]
		].
	^ tmpPackage! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 1/29/2012 21:25'!
registerInterestToAnnouncer: anAnnouncer
	"self registerInterestTo Announcement"

	"RPackageOrganizer registerInterestTo Announcement"

	" Announcer initialize"

	anAnnouncer
		on: CategoryAdded send: #systemCategoryAddedActionFrom: to: self;
		on: CategoryRemoved send: #systemCategoryRemovedActionFrom: to: self;
		on: CategoryRenamed send: #systemCategoryRenamedActionFrom: to: self;
		on:  ClassAdded send: #systemClassAddedActionFrom: to: self;
		on:  ClassCommented do: [:ann | 
				 "Only re-emit public corresponding announcement"
			"Not yet implemented?"
			];
		on:  ClassRecategorized send: #systemClassRecategorizedActionFrom: to: self;
		on:  ClassRemoved send: #systemClassRemovedActionFrom: to: self;
		on:  ClassRenamed send: #systemClassRenamedActionFrom: to: self;
		on:  ClassReorganized send: #systemClassReorganizedActionFrom: to: self;
		on:  MethodAdded send: #systemMethodAddedActionFrom: to: self;
		on:  MethodModified send: #systemMethodModifiedActionFrom: to: self;
		on:  MethodRecategorized send: #systemMethodRecategorizedActionFrom: to: self;
		on:  MethodRemoved send: #systemMethodRemovedActionFrom: to: self;
		on:  ProtocolAdded send: #systemProtocolAddedActionFrom: to: self;
		on:  ProtocolRemoved send: #systemProtocolRemovedActionFrom: to: self
		! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'IgorStasenko 8/12/2011 16:36'!
registerInterestToSystemAnnouncement
	"self registerInterestToSystemAnnouncement"

	self registerInterestToAnnouncer: self announcer! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 5/30/2011 16:52'!
systemCategoryAddedActionFrom: ann

	| rPackage packageSymbol |
	packageSymbol := ann item asSymbol.
	(self includesPackageNamed: packageSymbol)
		ifFalse: [ 
			rPackage := RPackage named: packageSymbol.
			self registerPackage: rPackage ]! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 5/30/2011 17:19'!
systemCategoryRemovedActionFrom: ann

	| rPackage categoryNameSymbol |
	categoryNameSymbol := ann categorySelector.
	"here we check that a corresponding MCPackage does not still exist. In this case we should keep the RPackage"
	MCWorkingCopy allManagers detect: [:each | self category: each packageName matches: categoryNameSymbol asString] ifNone: [
		rPackage := packages at: categoryNameSymbol ifAbsent: [^ self].
		"if no MCPackage exist, we unregister the package from the RPackageOrganizer"
		self basicUnregisterPackageNamed: categoryNameSymbol.	
		].
	
! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 5/27/2011 12:17'!
systemCategoryRenamedActionFrom: ann

	| rPackage classesAndProtocolsToRename|
	"we update the rpackage itself with the new name"
	rPackage := self packageNamed: ann oldNameSymbol ifAbsent: [RPackage new].
	rPackage name: ann newName.
	
	"we update the organizer"
	packages at: ann newName asSymbol put: rPackage.
	packages removeKey: ann oldNameSymbol ifAbsent: [self reportBogusBehaviorOf:   #systemCategoryRenamedActionFrom: ].
	self flag: #cyrille. "Should we not update the backPointer mapping too?"
	"CD: What do you mean by backpointerMapping? By telling a RPackage to rename itself, it should update itself all its internal. 
	Then it just left all references in the organizer to update. For now I see the 'packages' variable. "
	
	"we also rename all the extension protocols in the system with the new name"
	classesAndProtocolsToRename := (rPackage extensionMethods collect: [:aCompiledMethod |
		aCompiledMethod methodClass -> aCompiledMethod category
		]) asSet.
	classesAndProtocolsToRename do: [:anAssociation |
		anAssociation  key organization silentlyRenameCategory:  anAssociation  value toBe: '*', ann newName
		].
		
! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 5/31/2011 15:47'!
systemClassAddedActionFrom: ann

	| classNameSymbol class rPackage categoryNameSymbol |
	classNameSymbol := ann itemClassSymbol.
	class := ann itemClass.
	categoryNameSymbol := class category.
	rPackage := packages at: categoryNameSymbol ifAbsent: [
		"a class may be defined for a package that does not yet exist. In this case we have to create first the new package"
		self registerPackage: (RPackage named: categoryNameSymbol).
		].		
	"when we add the class, the parent category should normally already be register correctly at the RPackageOrganizer"
	"this next line should import the class and all its methods: "
	rPackage importClass: class.
	rPackage importClass: class theMetaClass
	"maybe the last update should be (is?) automatically done when adding or removing from a RPackage."
	"self default registerPackage: rPackage forClass: class"
	"maybe have to check for class and metaClass that will maybe both throw an event?
	=> seems to be correctly handled by RPackage"! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 5/31/2011 15:30'!
systemClassRecategorizedActionFrom: ann

	| class newRPackage oldRPackage newCategoryName oldCategoryName|
	"self flag: 'seems to be ok'."
	class := ann itemClass.	
	newCategoryName := ann newCategory asSymbol.
	oldCategoryName := ann oldCategory asSymbol.
	oldRPackage := self packageNamed: oldCategoryName.
	newRPackage := self packageNamed: newCategoryName ifAbsent: [
		"temporary code. For now adding a new category in the system does not emit correctly the corresponding event. The organizer is therefore not able to update itself. So here, if we don't find a matching package, we create a new one"
		self registerPackage: (RPackage named: newCategoryName)
		].
	"we have to update the old package"
	oldRPackage removeClass: class.	
	"we have to update the new package"
	newRPackage importClass: class.
	"we should also import the metaClass"
	newRPackage importClass: class theMetaClass	
	
	
	"we have to update the RPackageOrganizer.
	maybe the last update should be automatically done when adding or removing from a RPackage
	"
	"self default unregisterPackage: oldRPackage forClass: class.
	self default registerPackage: newRPackage forClass: class"! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 5/7/2011 22:19'!
systemClassRemovedActionFrom: ann
	"A class has been removed, we should update the package adequately."
	
	| class rPackage categoryName |	
	class := ann removedClass.
	categoryName := ann removedClassCategory.	
	rPackage := packages at: categoryName ifAbsent: [	^ self].
		
	"we have to update the rPackage and unregister the class and all its methods from it"
	rPackage removeClass: class.
	"we have to update the organizer:
             => update the 'classPackageDictionary' to remove the class"
	"maybe the last update should be automatically done when adding or removing from a RPackage => DONE"
	"self default unregisterPackage: rPackage forClass: class"
	
	self flag: #cyrille. "we have to remove all the extending method too. if the class is removed from the
	system then its extending methods should be removed too."
	"CD: Removing a class will remove the methods inside. Then removing a method should emit an announcement and the job will be done when catching this announcement"
	"SD- I do not think that we should raise a new event. In fact we should code a better remove method on RPackageOrganizer that performs well all the steps"! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 5/8/2011 15:52'!
systemClassRenamedActionFrom: ann
	"When a class is renamed, we update its package as well as the organizer"
	
	| newName oldName class rPackage extendingRPackages packageName |
	class := ann itemClass.
	packageName := ann itemPackageName.
	oldName := ann oldName.
	newName := ann newName.
	rPackage := self packageNamed: packageName.
	extendingRPackages := self extendingPackagesOfClassNamed: oldName.
	
	rPackage updateDefinedClassNamed: oldName withNewName: newName.
	"we have to update all RPackages extending this class"
	extendingRPackages do: [:aRPackage | 
		aRPackage updateExtensionForClassNamed: oldName withNewName: newName].
		
	"we have to update the RPackageOrganizer.
	update the 'classPackageDictionary' to replace the key with the new class name"
	self unregisterPackage: rPackage forClassName: oldName.
	self registerPackage: rPackage forClassName: newName.	
		
	"update the 'classExtendingPackagesMapping' to replace the key with the new class name"
	extendingRPackages do: [:aRPackage |
		self unregisterExtendingPackage: aRPackage forClassName: oldName.
		self registerExtendingPackage:  aRPackage forClassName: newName   
		].
	self flag: #cyrille. "we will see that."
	"maybe we should remove also the key from the extendedPackages dictionary"
	"CD: This is what are doing the lines above I think"

	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 5/10/2011 13:46'!
systemClassReorganizedActionFrom: ann
	"when a class is reorganized, we have to check if an extension has not been added"
	
	|class|
	self flag: 'Not sure this method is usefull. If we remove a category, it seems that we will remove the methods inside, therefore the remove method announcement will do the job. Then all other modifications does not impact the packages'.
	"CD: All tests pass with this version"
	"The event ReorganizedEvent is lauched for several cases:
		- we rename a protocol (see 'renameCategory:toBe: '). if The category is not empty, SystemMethodRecategorizedAnnouncement will also be emitted // THE WORK WILL BE DONE BY  SystemMethodRecategorizedAnnouncement
		- we sort (modify the order of) protocols (see 'sortCategories') // WE DON'T CARE HERE
		- we add a protocol (see 'addCategory:before:') // WE DON'T CARE HERE'
		- we remove a protocol (see 'removeCategory:' 'removeEmptyCategories') // THE WORK SHOULD BE DONE BY  SystemMethodRemovedAnnouncement
	".
! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 5/10/2011 13:42'!
systemClassReorganizedActionFromBis: ann
	"only to keep old code somewhere"
	"when a class is reorganized, we have to check if an extension has not been added"
	
	|class|
	self flag: 'Not sure this method is usefull. If we remove a category, it seems that we will remove the methods inside, therefore the remove method announcement will do the job. Then all other modifications does not impact the packages'.
	"The event ReorganizedEvent is lauched for several cases:
		- we rename a protocol (see 'renameCategory:toBe: '). if The category is not empty, SystemMethodRecategorizedAnnouncement will also be emitted // THE WORK WILL BE DONE BY  SystemMethodRecategorizedAnnouncement
		- we sort (modify the order of) protocols (see 'sortCategories') // WE DON'T CARE HERE
		- we add a protocol (see 'addCategory:before:') // WE DON'T CARE HERE'
		- we remove a protocol (see 'removeCategory:' 'removeEmptyCategories') // THE WORK SHOULD BE DONE BY  SystemMethodRemovedAnnouncement
		
		
	So the work here is quite big :) The event only provide a reference to the class that has been modified. So I guess we have to:
		=> we check that each extending rPackage contain all methods from the corresponding extension category (as extension methods)
		=> the parentPackage of the class includes all methods defined in classic categories
	".
	class := ann itemClass.
	class organization categories do: [:aString |
		(aString beginsWith: '*')
			ifTrue: [
				"here we deal with extension-categories"
				|extendingPackageName extendingPackage|
				extendingPackageName := aString copyWithout: $*.
				extendingPackage := self packageMatchingExtensionName: extendingPackageName.
				extendingPackage ifNotNil: [
					"we check that all extension method for this classs in the rpackage correspond to all methods in the extension category"
					(class methodsInCategory: aString) do: [:aMethodsSelector |
						"aMethodSelector can come from a trait used by class. Such a method should always belong to to the trait and therefore to the parent package of the trait"
					(class >> aMethodsSelector) isProvidedByATrait ifTrue: [^self].
						((extendingPackage extensionMethodsForClass: class) includes: (class methodDict at: aMethodsSelector))
							ifFalse: [
								"if a method has not been yet registered in the rPackage, we add it"
								((class methodDict at: aMethodsSelector) packageFromOrganizer: self ) removeMethod: (class methodDict at: aMethodsSelector).
								extendingPackage addMethod: (class methodDict at: aMethodsSelector).
								]  
						]
					]
				]
			ifFalse: [
				"here we deal with classic categories"
				(class methodsInCategory: aString) do: [:aMethodSelector |
					"aMethodSelector can come from a trait used by class. Such a method should always belong to to the trait and therefore to the parent package of the trait"
					(class traitOrClassOfSelector: aMethodSelector) = class ifFalse: [^self].
					((class package definedMethodsForClass: class) includes: (class methodDict at: aMethodSelector ) )
						ifFalse: [
							((class methodDict at: aMethodSelector) packageFromOrganizer: self) removeMethod: (class methodDict at: aMethodSelector).
							class package addMethod: (class methodDict at: aMethodSelector)
							]
					]
				]
			
		].
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'CamilloBruni 4/27/2012 17:50'!
systemMethodAddedActionFrom: ann

	|class selector protocol method rPackage |
	class := ann itemClass.
	protocol := ann itemProtocol.
	selector := ann itemSelector.
	method := ann itemMethod.
	"the method can come from a trait used by class. Such a method should always belong to to the trait and therefore to the parent package of the trait"
	method isProvidedByATrait ifTrue: [^ self].
	"we have to register the method in the parent RPackage of the class. 
	to do that, we first have to look if the method is an extension from an external package:"
	"1) in some cases, it can happen that protocol is nil (I do not know yet when). That's why we first check if protocol is nil. If it is, we just add the method worrying about anything. 
	This case can happen for exemple when running MCMethodDefinitionTest >> #testLoadAndUnload. Comment the nil check and should get an error."
	"This case come from CompiledMethod >> packageFromOrganizer:"
	
	rPackage := (self packageForProtocol: protocol inClass: method methodClass) ifNil: [
		"If the extension name does not match with any registered package, we create a new one, and add the method in this one. This is how monticello does."
		"class package"
		self registerPackage: (RPackage named: (protocol copyWithout: $*)).
		].  
	rPackage addMethod: method.
	
	
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'CamilloBruni 4/27/2012 17:52'!
systemMethodModifiedActionFrom: ann

	"this annoucement can correspond to several kind of modifications:
		=> a method has been renamed
		=> a method has been move to another category // SHOULD BE TREATED WITH SystemMethodRecategorizedEvent
		(
			-maybe from the classic category to an extending package
			-maybe from an extending package to another extending package
			-maybe from an extending package to a classic category
			- maybe from a classic category to another classic category
			)
	"
	| newProtocol oldProtocol oldMethod newMethod methodPackage destinationPackage |
	newMethod := ann newMethod.
	oldMethod := ann oldMethod.
	newProtocol := ann itemNewProtocol.
	oldProtocol := ann itemOldProtocol.
	self flag: #cyrille. 
	"This is not clear that we need to do soemthing here since the method is removed and added so the other events should be handled correctly
	We should remove this method to see if this is working."
	newMethod isProvidedByATrait ifTrue: [ ^ self ].
	methodPackage := newMethod packageFromOrganizer: self.
	"maybe the name of the method has changed"
	oldMethod selector = newMethod selector
		ifFalse: [ methodPackage updateSelector: oldMethod selector inClass: oldMethod methodClass withNewSelector: newMethod selector].
		

	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 5/13/2011 13:50'!
systemMethodRecategorizedActionFrom: ann
	"a protocol has been renamed
	a method has been removed. // WILL BE TREATED WITH SystemMethodRemovedAnnouncement
	a method has been moved to another category 
		-maybe from the classic category to an extending package
		-maybe from an extending package to another extending package
		-maybe from an extending package to a classic category
		- maybe from a classic category to another classic category"
		
	|oldProtocol newProtocol method methodPackage destinationPackage|
	self flag: #cyrille.
	oldProtocol := ann itemOldProtocol.
	"old protocol might be nil:"
	oldProtocol asString ifNil: [oldProtocol := ''].
	newProtocol := ann  itemNewProtocol.
	method := ann item.
	
	"1). See compiledMethod >> packageFromOrganizer:"
	
	"we have to pay attention that this announcement can come from TraitDescription>>updateOrganizationSelector:oldCategory:newCategory: . In this context, it can happen that the method referenced by the event is nil. In this case I think we should change nothing"
	
	
	method isNil ifFalse: [
		"the method can come from a trait used by class. Such a method should always belong to to the trait and therefore to the parent package of the trait"
		method isProvidedByATrait ifTrue: [^self].
		methodPackage := method packageFromOrganizer: self.
		"if newProtocol is nil, it means that we are in a 'methodRemoved' case. Nothing should be modified (all the work will be done when catching methodRemovedAnnouncement) " 
		newProtocol isNil ifTrue: [^self].
		"it can happen that oldProtocol is Nil. in this case we should do nothing"
		oldProtocol ifNil: [^ self].
		"maybe the category has changed"
		newProtocol asLowercase = oldProtocol asLowercase
			ifFalse: [
				"according the extension specified is valid or not, the method will be defined as an extension of an external package or in the class parent package"
				destinationPackage := (self packageForProtocol: newProtocol inClass: method methodClass) ifNil: [method methodClass package].
				methodPackage := (self packageForProtocol: oldProtocol inClass: method methodClass) ifNil: [method methodClass package].
				"then, we will remove the method from the method package, and add it to the destination package (we check before that the source package and the destination package are different)."
				methodPackage = destinationPackage
					ifFalse: [
						methodPackage removeMethod: method.
						destinationPackage addMethod: method
						]
				]
		]
	
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'CamilloBruni 4/27/2012 17:51'!
systemMethodRemovedActionFrom: ann

	| method methodPackage |
	method := ann itemMethod.
	method isProvidedByATrait ifTrue: [^self].
	methodPackage := method packageFromOrganizer: self.
	self flag: #cyrille.
	"is it clear that we have always a package"
	methodPackage removeMethod: method
	
	
	
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/18/2010 11:42'!
systemProtocolAddedActionFrom: ann

	"for now I don't see anything to do'"! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 5/13/2011 13:53'!
systemProtocolRemovedActionFrom: ann
	"to be unregistered"
	
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 4/13/2011 21:16'!
systemTraitDefinitionModifiedActionFrom: ann
	"this kind of anounecement will be used when:
		=> we create a new trait.
		=> we declare new traits to use in a trait"
	|trait|
	trait := ann itemTrait.
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'sd 10/15/2011 15:35'!
unregisterInterestToSystemAnnouncement
	"self unregisterInterestToSystemAnnouncement"

	SystemAnnouncer unsubscribe: self! !

Object subclass: #PackageChecker
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage-SystemIntegration'!

!PackageChecker methodsFor: 'check' stamp: 'StephaneDucasse 7/30/2010 14:03'!
check
	"self new check"
	
	"only work for top level package overlapping.
	Foo and Foo-Core, but not Foo-Core and Foo-Core-Test"
	| packageNames dict overlappingPackages |
	overlappingPackages := OrderedCollection new.
	packageNames := MCWorkingCopy allManagers collect: [:each | each packageName].
	dict := packageNames groupedBy: [:each | each copyUpTo: $-].
	dict copy keysAndValuesDo: [:k :v | v size = 1 ifTrue: [ dict removeKey: k]].
	dict keysAndValuesDo: [:k :v | (v includes: k) 
									ifTrue: [
										overlappingPackages add: v.
										Warning signal: 'Overlapping packages: ', v printString]].
	^ overlappingPackages ! !

!MCPackageManager class methodsFor: '*rpackage-systemintegration' stamp: 'StephaneDucasse 4/29/2011 20:40'!
unregisterForNotifications
	Smalltalk globals
		at: #SystemChangeNotifier
		ifPresent:
			[:cls|
			cls uniqueInstance noMoreNotificationsFor: self]
		! !

!ClassOrganizer methodsFor: '*rpackage-systemintegration' stamp: 'cyrilledelaunay 5/23/2011 11:47'!
silentlyRenameCategory: oldCatString toBe: newCatString
	| oldCat newCat oldElementsBefore oldElementsAfter index |
	oldCat := oldCatString asSymbol.
	newCat := newCatString asSymbol.
	oldElementsBefore := self listAtCategoryNamed: oldCat.
	(index := categoryArray indexOf: oldCat) = 0
		ifTrue: [^ self].	"old name not found, so no action"
	(categoryArray indexOf: newCat) > 0
		ifFalse: [
			categoryArray := categoryArray copy.  "need to change identity so smart list update will notice the change"
			categoryArray at: index put: newCat
			]
		ifTrue: [
			"if the category already exist, we move all elements inside and remove the old category"
			SystemChangeNotifier uniqueInstance doSilently: [
				oldElementsBefore do: [:anElement |
					self classify: anElement under: newCat
					].
			self removeCategory:  oldCat   				
				].
			].! !

TestCase subclass: #RPackageMCSynchronisationTest
	instanceVariableNames: 'emptyOrganizer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage-SystemIntegration'!

!RPackageMCSynchronisationTest commentStamp: '<historical>' prior: 0!
I had some problems using the 'ensure:' method. There was some conflicts with the organizer used for the test when trying to clean the classes, categories and packages generated for the test. 
There is something I don't get with this method. Therefore I choosed to clean by default all possible generated classes, categories and packages after each test using the tearDown method.

For now, all that was in the 'ensure:' methods has been commented.
Look for example at:
    testAddMethodInClassicCategoryAddMethodToTheParentPackageOfItsClass
and try to uncomment it!

!RPackageMCSynchronisationTest methodsFor: 'utilities' stamp: 'StephaneDucasse 4/24/2011 23:23'!
addXCategory

	Smalltalk globals organization addCategory: 'XXXXX'.
			! !

!RPackageMCSynchronisationTest methodsFor: 'utilities' stamp: 'StephaneDucasse 4/29/2011 21:44'!
addXYCategory

	self addXCategory.
	self addYCategory.
			! !

!RPackageMCSynchronisationTest methodsFor: 'utilities' stamp: 'StephaneDucasse 4/29/2011 21:50'!
addXYZCategory

	self addXCategory.
	self addYCategory.
	self addZCategory.
			! !

!RPackageMCSynchronisationTest methodsFor: 'utilities' stamp: 'StephaneDucasse 4/24/2011 23:26'!
addYCategory

	Smalltalk globals organization addCategory: 'YYYYY'.
			! !

!RPackageMCSynchronisationTest methodsFor: 'utilities' stamp: 'StephaneDucasse 4/29/2011 21:44'!
addZCategory

	Smalltalk globals organization addCategory: 'ZZZZZ'.
			! !

!RPackageMCSynchronisationTest methodsFor: 'setup' stamp: 'cyrilledelaunay 5/23/2011 11:18'!
cleanClassesPackagesAndCategories

		|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			Smalltalk removeClassNamed: 'RPackageNewStubClass'.
			Smalltalk removeClassNamed: 'RPackageOldStubClass'.
			Smalltalk removeClassNamed: 'Foo'.
			Smalltalk removeClassNamed: 'FooOther'.
			Smalltalk removeClassNamed: 'NewTrait'.
			Smalltalk removeClassNamed: 'ClassInYPackage'.
			Smalltalk removeClassNamed: 'ClassInZPackage'.
			Smalltalk organization removeCategory: 'Zork'.
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'XXXX'.
			Smalltalk organization removeCategory: 'YYYYY'.
			Smalltalk organization removeCategory: 'ZZZZZ'.
			Smalltalk organization removeCategory: 'FooPackage-Core'.
			Smalltalk organization removeCategory: 'FooPackage-Other'.
			Smalltalk organization removeCategory: 'FooPackage'.
			Smalltalk organization removeCategory: 'OriginalCategory'.
			Smalltalk organization removeCategory: 'NewCategoryName'.
			Smalltalk organization removeCategory: 'Y'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'OriginalCategory'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Yyyyy'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].	
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YyYyY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Y'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'ZZZZZ'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].	
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Zzzzz'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'FooPackage-Core'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'FooPackage-Other'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'FooPackage'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Zork'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			! !

!RPackageMCSynchronisationTest methodsFor: 'private' stamp: 'cyrille.delaunay 11/25/2010 13:17'!
createMethodNamed: methodName inClass: aClass inCategory: aCategoryName 

	^ aClass compile: (methodName, ' ^nil') classified: aCategoryName.! !

!RPackageMCSynchronisationTest methodsFor: 'private' stamp: 'cyrille.delaunay 11/25/2010 13:17'!
createMethodNamed: methodName inClassSideOfClass: aClass inCategory: aCategoryName 

	^ aClass classSide compile: (methodName, ' ^nil') classified: aCategoryName.! !

!RPackageMCSynchronisationTest methodsFor: 'private' stamp: 'StephaneDucasse 7/29/2010 23:46'!
createNewClassNamed: aName
	
	| cls |
	cls := Object subclass: aName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'RPackageTest'.
	^ cls! !

!RPackageMCSynchronisationTest methodsFor: 'private' stamp: 'cyrille.delaunay 11/22/2010 10:56'!
createNewClassNamed: aName inCategory: aCategoryName
	
	| cls |
	cls := Object subclass: aName asSymbol
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: aCategoryName.
	^ cls! !

!RPackageMCSynchronisationTest methodsFor: 'private' stamp: 'cyrille.delaunay 11/26/2010 15:31'!
createNewTraitNamed: aName inCategory: aCategoryName
	
	| trait |
	trait := Trait named: aName asSymbol
			uses: {}
			category: aCategoryName.
	^ trait! !

!RPackageMCSynchronisationTest methodsFor: 'accessing' stamp: 'RPackage 1/20/2011 13:17'!
emptyOrganizer 

	^ emptyOrganizer! !

!RPackageMCSynchronisationTest methodsFor: 'setup' stamp: 'cyrilledelaunay 5/30/2011 16:51'!
setUp 
	"by only using the 'default' RMockEmptyPackageOrganizer, we make sure that each time a new intance of the organizer is created, we unregister safely the last one from teh announcer (see the code of resetDefault). Like that, this old instance will not impact the system anymore when an event will be launched"
	
	RMockSystemAnnouncer initialize.
	emptyOrganizer := RPackageOrganizer new debuggingName: 'MCSynchronisation Package Organizer'.
	emptyOrganizer registerPackage: (RPackage named: 'as yet unclassified').
	
	" commented it since we are using withOrganizer:do: 
	emptyOrganizer registerInterestToAnnouncer: RMockSystemAnnouncer mockAnnouncer."
	"we also register interest to monticello, to be able to test the synchronisation between it and RPackage"
	emptyOrganizer registerInterestToMC. 
	"we have to unregister Monticello from system events to avoid errors when making our changes. Indeed, by default monticello make some request to the default organizer when changes are made in the system. But This default organizer does not register to system events during our tests"
	MCPackageManager unregisterForNotifications. 
	MCWorkingCopy unregisterForNotifications.
	"This is a second step of security. If by any chance, an Organizer instance persist in the system and is still registered to the (mock)announcer, we also control the (mock)announcer to be active only during the test "
	" commented it since we are using withOrganizer:do: 
	RMockSystemAnnouncer activateAnnouncerForSystemChanges."
	"we will create classes in those tests, we set the author name to avoid to have warning when running the tests"
	Author fullName ifNil: [Author fullName: 'RPackage'].! !

!RPackageMCSynchronisationTest methodsFor: 'setup' stamp: 'StephaneDucasse 10/16/2011 20:56'!
tearDown
	"If after a test, an orgizer is left somewhere, still registered at the mockAnnouncer, we don't want it to impact the system after the test. By disabling the MockSystemAnnouncer, nothing will happen when any kind of event will be lanched from the system"
	
	RMockSystemAnnouncer disableAnnouncerForSystemChanges.	
	RMockSystemAnnouncer mockAnnouncer unsubscribe: self emptyOrganizer.
	self emptyOrganizer unregisterInterestToMC.
	MCWorkingCopy removeDependent: self emptyOrganizer.
	self cleanClassesPackagesAndCategories.
	Author fullName ifNil: [Author fullName: nil].! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on categories' stamp: 'StephaneDucasse 4/29/2011 21:45'!
testAddCategoryCreatesPackage
	"test that when we add a category that does not yet correspond to an existing package, a new package is created"
	"self debug: #testAddCategoryCreatesPackage"
	
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer | 
			self addXCategory.
			self assert: (anOrganizer includesPackageNamed: #XXXXX)]
		] ensure: [self toDoAtTheEnd]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on categories' stamp: 'StephaneDucasse 4/29/2011 22:27'!
testAddCategoryIsAlreadyAPackageDoesNotCreatePackage
	"test that when we add a category corresponding to an existing package, we do not create a second package"
	"self debug: #testAddCategoryIsAlreadyAPackageDoesNotCreatePackage"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ 
			|tmpPackage|
			self addXCategory.
			tmpPackage := RPackage organizer packageNamed: #XXXXX.
			self addXCategory.
			self assert: tmpPackage =  (RPackage organizer packageNamed: #XXXXX) ]
		] ensure: [ self toDoAtTheEnd ]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 4/29/2011 21:46'!
testAddClassAddItIntoPackage
	"self debug: #testAddCategoryCreatesPackage"

	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|tmpPackage class|
			self addXCategory.
			self assert: (anOrganizer includesPackageNamed: #XXXXX).
			tmpPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self assert: (tmpPackage definesClass: class). 
			self assert: (class package = tmpPackage) ]
		] ensure: [self toDoAtTheEnd].
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 4/29/2011 21:46'!
testAddClassInNotCreatedPackage
	"test that when we add a class in an existing category, it does not create a new package"
	"self debug: #testAddClassInNotCreatedPackage"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|tmpPackage class|
			self addXCategory.
			self assert: (anOrganizer includesPackageNamed: #XXXXX).
			tmpPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self assert: (anOrganizer packageNamed: #XXXXX) = tmpPackage. 
			self assert: (class package = tmpPackage) ]
		] ensure: [ self toDoAtTheEnd ].! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 4/24/2011 23:24'!
testAddClassUpdateTheOrganizer
	"test that when we add a class, the organizer 'classPackageMapping' dictionary is updated, so that the class is linked to its package and so that we can access its parent package"
	"self debug: #testAddClassUpdateTheOrganizer"
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|tmpPackage class|
			self addXCategory.
			self assert: (anOrganizer includesPackageNamed: #XXXXX).
			tmpPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self assert: (anOrganizer packageOf: class) = tmpPackage. 
			]
		] ensure: [self toDoAtTheEnd.]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on MCPackages' stamp: 'cyrilledelaunay 5/4/2011 14:31'!
testAddMCPackageCreatesAPackage
	"test that when we create a MCPackage, a corresponding package is created"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			MCWorkingCopy forPackage: (MCPackage new name: #Zork).
			self assert: (anOrganizer includesPackageNamed: #Zork).
			] 
		] ensure: [ self toDoAtTheEnd.]
		
					  ! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on MCPackages' stamp: 'StephaneDucasse 4/24/2011 23:08'!
testAddMCPackageForCategoryAlreadyExistingDoesNotCreateAPackage
	"test that when we create a MCPackage and that a category of this name already exists, no package is created"
	"self debug: #testAddMCPackageForCategoryAlreadyExistingDoesNotCreateAPackage"
	"self run: #testAddMCPackageForCategoryAlreadyExistingDoesNotCreateAPackage"	
	
	[ RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|tmpPackage|
			Smalltalk globals organization addCategory: 'Zork'.
			tmpPackage := anOrganizer packageNamed: #Zork.
			MCWorkingCopy forPackage: (MCPackage new name: #Zork).
			self assert: tmpPackage = (anOrganizer packageNamed: #Zork).
			] 
		] ensure: [ self toDoAtTheEnd.]
		
		
					  ! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'StephaneDucasse 4/29/2011 21:47'!
testAddMethodByUsingATraitDoesNotAddTheMethodToThePackage
	"test that when a method is added to a class bu using a trait, this method is not imported in the parent package of the class"
	"self debug: #testAddMethodByUsingATraitDoesNotAddTheMethodToThePackage"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class trait|
			self addXCategory.
			self addYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			trait := self createNewTraitNamed: 'NewTrait' inCategory: 'YYYYY'.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass:  trait inCategory: 'trait category'.
			
			self assert: (XPackage methods isEmpty ).
			self deny: (YPackage methods isEmpty ).
			class addToComposition: trait.
			self assert: (YPackage includesDefinedSelector: #stubMethod ofClass: trait).
			self assert: (XPackage methods isEmpty).]
		] ensure: [ self toDoAtTheEnd .]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'StephaneDucasse 4/24/2011 23:25'!
testAddMethodInClassicCategoryAddMethodToTheParentPackageOfItsClass
	"test that when we add a method to a  class in a classic category (*not beginning with *), this method is added to the parent package of the class"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|tmpPackage class|
			self addXCategory.
			self assert: (anOrganizer includesPackageNamed: #'XXXXX').
			tmpPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass:  class inCategory: 'classic category'.
			self assert: (class package includesDefinedSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = class package.
			]
		] ensure: [ self toDoAtTheEnd]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'StephaneDucasse 4/29/2011 21:47'!
testAddMethodInClassicCategoryAddMethodToTheParentPackageOfItsTrait
	"test that when we add a method to a  trait in a classic category (*not beginning with *), this method is added to the parent package of the class"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|tmpPackage trait|
			self addXCategory.
			self assert: (anOrganizer includesPackageNamed: #'XXXXX').
			tmpPackage := anOrganizer packageNamed: #XXXXX.
			trait := self createNewTraitNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass:  trait inCategory: 'classic category'.
			self assert: (tmpPackage includesClass: trait).
			self assert: (trait package includesDefinedSelector: #stubMethod ofClass: trait).
			self assert: ((trait >> #stubMethod) packageFromOrganizer: anOrganizer) = trait package ]
		] ensure: [ self toDoAtTheEnd.]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'StephaneDucasse 4/29/2011 21:48'!
testAddMethodInExtensionCategoryAddMethodToTheExtendingPackage
	"test that when we add a method to a  class in an extension category ( beginning with *), this method is added to the corresponding extending package"
	
	self flag: #cyrille. "what are these haltif???? in a test????"
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			| class firstPackage secondPackage|
			self addXYCategory.		
			firstPackage := anOrganizer packageNamed: #XXXXX.
			secondPackage := anOrganizer packageNamed: #YYYYY.
			self haltIf: [(RPackage organizer =  anOrganizer) not].
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self assert: (class package = firstPackage).
			self haltIf: [(RPackage organizer =  anOrganizer) not].
			self createMethodNamed: 'stubMethod' inClass: class inCategory: '*YYYYY'.
			self haltIf: [(RPackage organizer =  anOrganizer) not].
			self assert: (secondPackage includesExtensionSelector: #stubMethod ofClass: class).
			self deny: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = secondPackage ]
		] ensure: [ self toDoAtTheEnd .]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'StephaneDucasse 4/29/2011 21:48'!
testAddMethodInExtensionCategoryEnlargingPackageNameAddMethodToTheExtendingPackage
	"test that when we add a method  in an extension category ( beginning with*) that enlarge a package name (for example *mondrian-accessing for Mondrian), this method is added to the corresponding extending package"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			| class XPackage YPackage|
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self assert: class package = XPackage.
			self createMethodNamed: #newMethod inClass:  class inCategory: '*YYYYY-subcategory'.
			self assert: (YPackage includesExtensionSelector: #newMethod ofClass: class).
			self deny: (XPackage includesDefinedSelector: #newMethod ofClass: class).
			self assert: ((class >> #newMethod) packageFromOrganizer: anOrganizer) = YPackage ]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'expectedfailures' stamp: 'StephaneDucasse 4/29/2011 21:20'!
testAddMethodInExtensionCategoryNotExistingCreateANewPackage
	"test that when we add a method  in an extension category ( beginning with *)that does not refer to an existing categorya new package with the name of this category is added, and that the method is added to this new package"
	
	self flag: #stef. "this is strange since the setup does not use withOrganizer:do: and now we use it. There are some inconsistencies"
	"not sure this is the behavior expected. Maybe a new package should be created and should contain the method. Actually, this is the reason the test fail :)."
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			| class firstPackage secondPackage|
			Smalltalk globals organization addCategory: 'XXXXX'.
			Smalltalk globals organization addCategory: 'YYYYY'.
			firstPackage := anOrganizer packageNamed: #XXXXX.
			secondPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: #newMethod inClass:  class inCategory: '*YYYY'.
			self deny: (secondPackage includesExtensionSelector: #newMethod ofClass: class).
			self deny: (class package includesDefinedSelector: #newMethod ofClass: class).
			self assert: ((anOrganizer packageNamed: #YYYY) = nil) not.
			self assert: ((class >> #newMethod) packageFromOrganizer: anOrganizer) = (anOrganizer packageNamed: #YYYY).
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'StephaneDucasse 4/29/2011 21:48'!
testAddMethodInExtensionCategoryNotRespectingCaseAddMethodToTheExtendingPackage
	"test that when we add a method  in an extension category ( beginning with *)thae does not match the case of the corresponding package (for example *packagea for PackageA), this method is added to the corresponding extending package"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			| class firstPackage secondPackage|
			self addXYCategory.
			firstPackage := anOrganizer packageNamed: #XXXXX.
			secondPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass:  class inCategory: '*yyYyY'.
			self assert: (secondPackage includesExtensionSelector: #stubMethod ofClass: class).
			self deny: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = secondPackage.
			]
		] ensure: [ self toDoAtTheEnd.]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on traits' stamp: 'StephaneDucasse 4/29/2011 21:22'!
testAddTraitAddItIntoPackage
	"self debug: #testAddCategoryCreatesPackage"

	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|tmpPackage trait|
			self addXCategory.
			self assert: (anOrganizer includesPackageNamed: 'XXXXX').
			tmpPackage := anOrganizer packageNamed: #XXXXX.
			trait := self createNewTraitNamed: 'NewTrait' inCategory: 'XXXXX'.
			self assert: (tmpPackage definesClass: trait). 
			]
		] ensure: [ self toDoAtTheEnd ].
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on traits' stamp: 'StephaneDucasse 4/29/2011 21:22'!
testAddTraitInNotCreatedPackage
	"test that when we add a trait in an existing category, it does not create a new package"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|tmpPackage trait|
			self addXCategory.
			self assert: (anOrganizer includesPackageNamed: 'XXXXX').
			tmpPackage := anOrganizer packageNamed: #XXXXX.
			trait := self createNewClassNamed: 'NewTrait' inCategory: 'XXXXX'.
			self assert: (anOrganizer packageNamed: #XXXXX) = tmpPackage. 
			self assert: (trait package = tmpPackage)
			]
		] ensure: [ self toDoAtTheEnd ].! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on traits' stamp: 'StephaneDucasse 4/29/2011 21:22'!
testAddTraitUpdateTheOrganizer
	"test that when we add a trait, the organizer 'classPackageMapping' dictionary is updated, so that the class is linked to its package and so that we can access its parent package"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|tmpPackage trait|
			self addXCategory.
			self assert: (anOrganizer includesPackageNamed: 'XXXXX').
			tmpPackage := anOrganizer packageNamed: #XXXXX.
			trait := self createNewTraitNamed: 'NewTrait' inCategory: 'XXXXX'.
			self assert: (anOrganizer packageOf: trait) = tmpPackage. 
			]
		] ensure: [ self toDoAtTheEnd]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'StephaneDucasse 4/24/2011 23:12'!
testBasicImportSystemClass
	"self debug: #testBasicImportSystemClass"
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer | 
			| fooClass fooPackage |
			fooClass :=  Object subclass: #Foo
							instanceVariableNames: ''
							classVariableNames: ''
							poolDictionaries: ''
							category: 'FooPackage'.
			fooClass compileSilently: 'foo ^ 1' classified: 'foo'.
			fooClass compileSilently: 'fooBar ^ 2' classified: 'foo'.
			fooClass compileSilently: 'fooFooBar ^ 3' classified: 'foo'.
			fooClass class compileSilently: 'fooClassSide ^ 1' classified: 'foo'.
			fooClass compileSilently: 'zork ^ 4' classified: '*zork'.
			fooPackage := RPackage named: #FooPackage.
			fooPackage importCategory.
			self assert: (fooPackage includesDefinedSelector: #foo ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooBar ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooFooBar ofClass: fooClass).
			self deny: (fooPackage includesDefinedSelector: #zork ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooClassSide ofClass: fooClass class).
			 ]
		] ensure: [ self toDoAtTheEnd.]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'StephaneDucasse 4/24/2011 23:12'!
testCategoryMatching
	
	[self assert: (RPackageOrganizer  default category: 'Compression-Archives' matches: 'Compression').
	self deny: (RPackageOrganizer default category: 'Compression' matches: 'Compression-Archives').
	self assert: (RPackageOrganizer default category: 'Compression' matches: 'Compression').
	self assert: (RPackageOrganizer default category: 'Compression-Arh' matches: 'Compression').
	self deny: (RPackageOrganizer default category: 'XXXX' matches: 'Compression-Archives').
		] ensure: [ self toDoAtTheEnd.]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'StephaneDucasse 4/24/2011 23:13'!
testImportSystallManagershSubCategoryAndAnotherCategory
	"self debug: #testImportSystemClassWithSubCategoryAndAnotherCategory"
	"indeed the user may have created a class in the category Foo-Core, and after created
	a package only named Foo. The classes in category Foo-Core should be added to Foo. But
	some other packages and categories starting with Foo may exist."
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer | 
			| fooClass fooPackage fooClass2|
			fooClass := self createNewClassNamed: 'Foo' inCategory: 'FooPackage-Core'.
			fooClass compileSilently: 'foo ^ 1' classified: 'foo'.
			fooClass compileSilently: 'fooBar ^ 2' classified: 'foo'.
			fooClass compileSilently: 'fooFooBar ^ 3' classified: 'foo'.
			fooClass class compileSilently: 'fooClassSide ^ 1' classified: 'foo'.
			fooClass compileSilently: 'zork ^ 4' classified: '*zork'.

			fooClass2 := self createNewClassNamed: 'FooOther' inCategory: 'FooPackage-Other'.
			fooClass2 compileSilently: 'fooOther ^ 1' classified: 'what s up'.
			fooClass2 compileSilently: 'fooBarOther ^ 2' classified: 'dr'.
			fooClass2 class compileSilently: 'fooOtherClassSide ^ 1' classified: 'lovely carots'.
			fooClass2 compileSilently: 'zorkZork ^ 4' classified: '*zorkzork'.
			
			fooPackage := RPackage named: #FooPackage.
			fooPackage importCategory.
			self assert: (fooPackage definedClasses size = 2).
			
			self assert: (fooPackage includesDefinedSelector: #foo ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooBar ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooFooBar ofClass: fooClass).
			self deny: (fooPackage includesDefinedSelector: #zork ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooClassSide ofClass: fooClass class).
			
			self assert: (fooPackage includesDefinedSelector: #fooOther ofClass: fooClass2).
			self assert: (fooPackage includesDefinedSelector: #fooBarOther ofClass: fooClass2).
			self deny: (fooPackage includesDefinedSelector: #zorkZork ofClass: fooClass2).
			self assert: (fooPackage includesDefinedSelector: #fooOtherClassSide ofClass: fooClass2 class).
			 ]
		] ensure: [ self toDoAtTheEnd]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'StephaneDucasse 4/24/2011 23:14'!
testImportSystemClassWithSubCategory
	"self debug: #testImportSystemClassWithSubCategory"
	"indeed the user may have created a class in the category Foo-Core, and after created
	a package only named Foo. The classes in category Foo-Core should be added to Foo. But
	some other packages and categories starting with Foo may exist."
	[
	RPackage 
		withOrganizer: self emptyOrganizer 		
		do: [ :anOrganizer | 
			| fooClass fooPackage |
			fooClass :=  Object subclass: #Foo
							instanceVariableNames: ''
							classVariableNames: ''
							poolDictionaries: ''
							category: 'FooPackage-Core'.
			fooClass compileSilently: 'foo ^ 1' classified: 'foo'.
			fooClass compileSilently: 'fooBar ^ 2' classified: 'foo'.
			fooClass compileSilently: 'fooFooBar ^ 3' classified: 'foo'.
			fooClass class compileSilently: 'fooClassSide ^ 1' classified: 'foo'.
			fooClass compileSilently: 'zork ^ 4' classified: '*zork'.
			fooPackage := RPackage named: #FooPackage.
			fooPackage importCategory.
			self assert: (fooPackage includesDefinedSelector: #foo ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooBar ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooFooBar ofClass: fooClass).
			self deny: (fooPackage includesDefinedSelector: #zork ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooClassSide ofClass: fooClass class).
			]] ensure: [ self toDoAtTheEnd.]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'StephaneDucasse 4/29/2011 21:23'!
testImportSystemClassWithSubCategoryAndAnotherCategory
	"self debug: #testImportSystemClassWithSubCategoryAndAnotherCategory"
	"indeed the user may have created a class in the category Foo-Core, and after created
	a package only named Foo. The classes in category Foo-Core should be added to Foo. But
	some other packages and categories starting with Foo may exist."
	
	[ RPackage 
		withOrganizer: self emptyOrganizer 
		do: [ :anOrganizer | 
			| fooClass fooPackage fooClass2|
			fooClass :=  Object subclass: #Foo
							instanceVariableNames: ''
							classVariableNames: ''
							poolDictionaries: ''
							category: 'FooPackage-Core'.
			fooClass compileSilently: 'foo ^ 1' classified: 'foo'.
			fooClass compileSilently: 'fooBar ^ 2' classified: 'foo'.
			fooClass compileSilently: 'fooFooBar ^ 3' classified: 'foo'.
			fooClass class compileSilently: 'fooClassSide ^ 1' classified: 'foo'.
			fooClass compileSilently: 'zork ^ 4' classified: '*zork'.
			
			fooClass2 :=  Object subclass: #FooOther
							instanceVariableNames: ''
							classVariableNames: ''
							poolDictionaries: ''
							category: 'FooPackage-Other'.
			fooClass2 compileSilently: 'fooOther ^ 1' classified: 'what s up'.
			fooClass2 compileSilently: 'fooBarOther ^ 2' classified: 'dr'.
			fooClass2 class compileSilently: 'fooOtherClassSide ^ 1' classified: 'lovely carots'.
			fooClass2 compileSilently: 'zorkZork ^ 4' classified: '*zorkzork'.
			
			fooPackage := RPackage named: #FooPackage.
			fooPackage importCategory.
			self assert: (fooPackage definedClasses size = 2).
			self assert: (fooPackage includesDefinedSelector: #foo ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooBar ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooFooBar ofClass: fooClass).
			self deny: (fooPackage includesDefinedSelector: #zork ofClass: fooClass).
			self assert: (fooPackage includesDefinedSelector: #fooClassSide ofClass: fooClass class).
			self assert: (fooPackage includesDefinedSelector: #fooOther ofClass: fooClass2).
			self assert: (fooPackage includesDefinedSelector: #fooBarOther ofClass: fooClass2).
			self deny: (fooPackage includesDefinedSelector: #zorkZork ofClass: fooClass2).
			self assert: (fooPackage includesDefinedSelector: #fooOtherClassSide ofClass: fooClass2 class).
			 ]] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'StephaneDucasse 4/29/2011 21:24'!
testModifyMethodByChangingCode
	"test that when we modify the code the code of a method, everything work well "
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			| class firstPackage secondPackage|
			self addXCategory.
			firstPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass:  class inCategory: 'classic category'.
		
			self assert: (class >> #stubMethod) category = 'classic category'. 
			self assert: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self deny: (firstPackage includesExtensionSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = firstPackage.
			
			class compile: 'stubMethod ^22222222222'.
			
			"nothing should change"
			self assert: (class >> #stubMethod) category = 'classic category'. 
			self assert: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self deny: (firstPackage includesExtensionSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = firstPackage.
			]
		] ensure: [ self toDoAtTheEnd]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'StephaneDucasse 4/29/2011 21:24'!
testModifyMethodByMovingFromClassicCategoryToClassicCategoryDoesNothing
	"test that when we move a method from a classic category (not begining with *) to another classic category , nothing is modified"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			| class firstPackage secondPackage|
			self addXCategory.
			firstPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass:  class inCategory: 'classic category'.
		
			self assert: (class >> #stubMethod) category = 'classic category'. 
			self assert: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self deny: (firstPackage includesExtensionSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = firstPackage.
			
			class organization classify: #stubMethod under: 'new category'.
			self assert: (class >> #stubMethod) category = 'new category'. 
			self assert: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self deny: (firstPackage includesExtensionSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = firstPackage ]
		] ensure: [ self toDoAtTheEnd]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'StephaneDucasse 4/29/2011 21:49'!
testModifyMethodByMovingFromClassicCategoryToExtensionCategoryMoveItFromParentPackageToExtendingPackage
	"test that when we move a method from a classic category (not begining with *) to an extension category , the method is moved from the parent package of the class to the extending package"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			| class firstPackage secondPackage|
			self addXYCategory. 
			firstPackage := anOrganizer packageNamed: #XXXXX.
			secondPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass:  class inCategory: 'classic category'.
		
			self assert: (class >> #stubMethod) category = 'classic category'. 
			self assert: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self deny: (firstPackage includesExtensionSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = firstPackage.
			
			class organization classify: #stubMethod under: '*yyyyy'.	
			self assert: (class >> #stubMethod) category = '*yyyyy'. 
			self deny: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self assert: (secondPackage includesExtensionSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = secondPackage.
			
			class organization classify: #stubMethod under: '*yyyyy-suncategory'.
			"class compileSilently: (class>>#stubMethod) getSource classified: '*yyyyy-suncategory'."
			self assert: (class >> #stubMethod) category = '*yyyyy-suncategory'. 
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = secondPackage.
			]
		] ensure: [ self toDoAtTheEnd.]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'StephaneDucasse 4/29/2011 21:49'!
testModifyMethodByMovingFromExtensionCategoryToClassicCategoryMoveItFromExtendingPackageToParentPackage

	"test that when we move a method from an extension category ( begining with *) to a classic category , the method is moved from the extending package to the parent package of the class"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			| class firstPackage secondPackage|
			self addXYCategory.
			firstPackage := anOrganizer packageNamed: #XXXXX.
			secondPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass:  class inCategory: '*yyyyy'.
		
			self assert: (class >> #stubMethod) category = '*yyyyy'. 
			self deny: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self assert: (secondPackage includesExtensionSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = secondPackage.
			
			class organization classify: #stubMethod under: 'classic category'.
			
			self assert: (class >> #stubMethod) category = 'classic category'. 
			self assert: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
			self deny: (secondPackage includesExtensionSelector: #stubMethod ofClass: class).
			self assert: ((class >> #stubMethod) packageFromOrganizer: anOrganizer) = firstPackage.
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'StephaneDucasse 4/29/2011 22:27'!
testModifyMethodByMovingFromExtensionCategoryToExtensionCategoryMoveItFromExtendingPackageToSecondExtendingPackage
	"test that when we move a method from an extension category ( begining with *) to another extending package , the method is moved from the extending package to the other extending package"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			| class XPackage YPackage ZPackage|
			self addXYZCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			ZPackage := anOrganizer packageNamed: #ZZZZZ.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: #newMethod inClass:  class inCategory: '*yyyyy'.
		
			self assert: (class >> #newMethod) category = '*yyyyy'. 
			self deny: (XPackage includesDefinedSelector: #newMethod ofClass: class).
			self assert: (YPackage includesExtensionSelector: #newMethod ofClass: class).
			self assert: ((class >> #newMethod) packageFromOrganizer: anOrganizer) = YPackage.
			
			class organization classify: #newMethod under: '*zzzzz'.
			self assert: (class >> #newMethod) category = '*zzzzz'. 
			self deny: (XPackage includesDefinedSelector: #newMethod ofClass: class).
			self deny: (YPackage includesExtensionSelector: #newMethod ofClass: class).
			self assert: (ZPackage includesExtensionSelector: #newMethod ofClass: class).
			self assert: ((class >> #newMethod) packageFromOrganizer: anOrganizer) = ZPackage.
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'cyrilledelaunay 5/16/2011 13:50'!
testRecategorizeClassInUnknownCategoryRegisterTheClassInANewPackage
	"test that when we recategorize a class in category not yet registered in RPackage, a new rPackage is created with the class inside"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class|
			self addXCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self assert: (anOrganizer packageOf: class) = XPackage. 
			
			class category: 'YYYYY'.
			YPackage := anOrganizer packageNamed: #YYYYY.
			self assert: (anOrganizer packageOf: class) = YPackage. 
			self deny: (anOrganizer packageOf: class) = XPackage. 
			]
		] ensure: [ self toDoAtTheEnd.]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 4/29/2011 21:26'!
testRecategorizeClassRegisterTheClassInTheNewPackage
	"test that when we recategorize a class, the new package in which it is defined include it"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class|
			self addXCategory.
			self addYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self assert: (anOrganizer packageOf: class) = XPackage. 
			
			class category: 'YYYYY'.
			self assert: (anOrganizer packageOf: class) = YPackage. 
			self deny: (anOrganizer packageOf: class) = XPackage. 
			]
		] ensure: [ self toDoAtTheEnd.]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 4/29/2011 21:27'!
testRecategorizeClassRegisterTheClassMethodsInTheNewPackage

	"test that when we recategorize a class, the new package in which it is defined include all the methods defined in this class (not extensions)"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage ZPackage class|
			self addXCategory.
			self addYCategory.
			Smalltalk organization addCategory: 'ZZZZZ'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			ZPackage := anOrganizer packageNamed: #ZZZZZ.
			
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'method1' inClass: class inCategory: 'category'.
			self createMethodNamed: 'method2' inClass: class inCategory: 'category'.
			self createMethodNamed: 'method3' inClass: class inCategory: '*zzzzz'.
			self createMethodNamed: 'method4' inClass: class inCategory: '*yyyyy'.
			
			self assert: (XPackage includesDefinedSelector: #method1 ofClass: class ). 
			self assert: (XPackage includesDefinedSelector: #method2 ofClass: class ).
			self assert: (ZPackage includesExtensionSelector: #method3 ofClass: class ). 
			self assert: (YPackage includesExtensionSelector: #method4 ofClass: class ). 
			
			class category: 'YYYYY'.
			
			self assert: (YPackage includesDefinedSelector: #method1 ofClass: class ). 
			self deny: (XPackage includesDefinedSelector: #method1 ofClass: class ). 
			self deny: (XPackage includesExtensionSelector: #method1 ofClass: class ). 
			self assert: (YPackage includesDefinedSelector: #method2 ofClass: class ).
			self deny: (XPackage includesDefinedSelector: #method2 ofClass: class ). 
			self deny: (XPackage includesExtensionSelector: #method2 ofClass: class ). 
			self deny: (YPackage includesExtensionSelector: #method3 ofClass: class ). 
			self deny: (YPackage includesDefinedSelector: #method3 ofClass: class ). 
			self assert: (ZPackage includesExtensionSelector: #method3 ofClass: class ). 
			"I am not sure about this test. For now it does not pass, but I don't know what is the expecting behavior here."
			"self assert: (YPackage includesExtensionSelector: #method4 ofClass: class ). "
			]
		] ensure: [ self toDoAtTheEnd ]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 4/29/2011 21:28'!
testRecategorizeClassUnregisterTheClassFromTheOldPackage
	"test that when we recategorize a class, the old package in which it was defined don't include it anymore"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class|
			self addXCategory.
			self addYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self assert: (anOrganizer packageOf: class) = XPackage. 
			
			class category: 'YYYYY'.
			self deny: (XPackage includesClass: class). 
			]
		] ensure: [ self toDoAtTheEnd ]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 4/29/2011 21:29'!
testRecategorizeClassUnregisterTheClassMethodsFromTheOldPackage
	"test that when we recategorize a class, the old package in which it was defined don't include its defined methods (not extensions) anymore"
	"self debug: #testRecategorizeClassUnregisterTheClassMethodsFromTheOldPackage"
		
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage ZPackage class|
			self addXCategory.
			self addYCategory.
			Smalltalk organization addCategory: 'ZZZZZ'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			ZPackage := anOrganizer packageNamed: #ZZZZZ.
			
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'method1' inClass: class inCategory: 'category'.
			self createMethodNamed: 'method2' inClass: class inCategory: 'category'.
			self createMethodNamed: 'method3' inClass: class inCategory: '*zzzzz'.
			self createMethodNamed: 'method4' inClass: class inCategory: '*yyyyy'.
			
			self assert: (XPackage includesDefinedSelector: #method1 ofClass: class ). 
			self assert: (XPackage includesDefinedSelector: #method2 ofClass: class ).
			self assert: (ZPackage includesExtensionSelector: #method3 ofClass: class ). 
			self assert: (YPackage includesExtensionSelector: #method4 ofClass: class ). 
			
			class category: 'YYYYY'.
			self deny: (XPackage includesDefinedSelector: #method1 ofClass: class ). 
			self deny: (XPackage includesExtensionSelector: #method1 ofClass: class ). 
			self deny: (XPackage includesDefinedSelector: #method2 ofClass: class ). 
			self deny: (XPackage includesExtensionSelector: #method2 ofClass: class ). 
			self deny: (XPackage includesDefinedSelector: #method3 ofClass: class ). 
			self deny: (XPackage includesExtensionSelector: #method3 ofClass: class ). 
			self deny: (XPackage includesDefinedSelector: #method4 ofClass: class ). 
			self deny: (XPackage includesExtensionSelector: #method4 ofClass: class ).
			]
		] ensure: [ self toDoAtTheEnd	]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 4/29/2011 21:30'!
testRecategorizeClassUpdateTheOrganizer
	"test that when we recategorize a class, the organizer is updated so that the class name point the the new RPackage"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class|
			self addXCategory.
			self addYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			class category: 'YYYYY'.	
			self assert: (anOrganizer packageOf: class) = YPackage
			]
		] ensure: [ self toDoAtTheEnd ]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 4/29/2011 21:31'!
testRecategorizeClassWithExtensionInTheExtendedPackage
	"test that when we recategorize a class with extension methods in the corresponding extended package, the extension methods become defined methods"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class|
			self addXCategory.
			self addYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: #RPackageTestsExtensionMethod inClass: class  inCategory: '*yyyyy'.  
			self assert: (anOrganizer packageOf: class) = XPackage. 
			self assert: (YPackage extensionSelectors includes: #RPackageTestsExtensionMethod). 
			self deny: ((XPackage definedSelectorsForClass: class) includes: #RPackageTestsExtensionMethod). 
		
			class category: 'YYYYY'.
			self assert: (anOrganizer packageOf: class) = YPackage. 
			self deny: (anOrganizer packageOf: class) = XPackage. 
			self deny: ((XPackage definedSelectorsForClass: class) includes: #RPackageTestsExtensionMethod).
			self assert: ((YPackage definedSelectorsForClass: class)  includes: #RPackageTestsExtensionMethod). 
			self deny: (YPackage extensionSelectors includes: #RPackageTestsExtensionMethod). 
			]
		] ensure: [ self toDoAtTheEnd ]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'cyrilledelaunay 5/31/2011 15:38'!
testRecategorizeClassWithMetaClassMethodsRegisterAllClassMethodsInTheNewPackage

	"test that when we recategorize a class (having methods defined in both instance and class side), the new package in which it is defined include all the methods (from instance and class side) defined in this class (not extensions)"
	
	| addedCount addedCat | 
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage ZPackage class|
			self addXCategory.
			self addYCategory.
			Smalltalk organization addCategory: 'ZZZZZ'.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			ZPackage := anOrganizer packageNamed: #ZZZZZ.
			
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'method1' inClass: class inCategory: 'category'.
			self createMethodNamed: 'method2' inClass: class inCategory: 'category'.
			self createMethodNamed: 'method3' inClass: class inCategory: '*zzzzz'.
			self createMethodNamed: 'method4' inClass: class inCategory: '*yyyyy'.
			self createMethodNamed: 'method5' inClass: class theMetaClass inCategory: 'category'.
			self createMethodNamed: 'method6' inClass: class theMetaClass inCategory: 'category'.
			self createMethodNamed: 'method7' inClass: class theMetaClass inCategory: '*zzzzz'.
			
			self assert: (XPackage includesDefinedSelector: #method1 ofClass: class ). 
			self assert: (XPackage includesDefinedSelector: #method2 ofClass: class ).
			self assert: (ZPackage includesExtensionSelector: #method3 ofClass: class ). 
			self assert: (YPackage includesExtensionSelector: #method4 ofClass: class ).
			self assert: (XPackage includesDefinedSelector: #method5 ofMetaclassName: class name). 
			self assert: (XPackage includesDefinedSelector: #method6 ofMetaclassName: class name). 
			self assert: (ZPackage includesExtensionSelector: #method7 ofMetaclassName: class name ). 
			
			class category: 'YYYYY'.
			
			self assert: (YPackage includesDefinedSelector: #method1 ofClass: class ). 
			self deny: (XPackage includesDefinedSelector: #method1 ofClass: class ). 
			self deny: (XPackage includesExtensionSelector: #method1 ofClass: class ). 
			self assert: (YPackage includesDefinedSelector: #method2 ofClass: class ).
			self deny: (XPackage includesDefinedSelector: #method2 ofClass: class ). 
			self deny: (XPackage includesExtensionSelector: #method2 ofClass: class ). 
			self deny: (YPackage includesExtensionSelector: #method3 ofClass: class ). 
			self deny: (YPackage includesDefinedSelector: #method3 ofClass: class ). 
			self assert: (ZPackage includesExtensionSelector: #method3 ofClass: class ). 
			"lets check metaclass methods"
			self assert: (YPackage includesDefinedSelector: #method5 ofMetaclassName: class name). 
			self assert: (YPackage includesDefinedSelector: #method6 ofMetaclassName: class name). 
			self deny: (XPackage includesDefinedSelector: #method5 ofMetaclassName: class name). 
			self deny: (XPackage includesDefinedSelector: #method6 ofMetaclassName: class name). 
			self assert: (ZPackage includesExtensionSelector: #method7 ofMetaclassName: class name ). 
			"I am not sure about this test. For now it does not pass, but I don't know what is the expecting behavior here."
			"self assert: (YPackage includesExtensionSelector: #method4 ofClass: class ). "
			]
		] ensure: [ self toDoAtTheEnd ]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'cyrilledelaunay 5/27/2011 11:04'!
testRemoveAllExtensionMethodsFromMetaAndInstanceSideUpdateOrganizerExtensionsDictionary
	"test that In a class (from the meta  and the instance side ), when we remove all the method extended by the same external package, the class get unregistered from the classExtendingPackagesMapping of the organizer "

	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class YPackage|
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage  := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'. 
			self createMethodNamed: 'stubMethod2' inClass: class theMetaClass  inCategory: '*yyyyy'. 
			
			self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
			self assert: (YPackage includesExtensionSelector: #stubMethod2 ofClass: class theMetaClass). 
			class removeSelector: #stubMethod.
			"there should be no differences made between class and metaClass:"
			self assert: ((anOrganizer extendingPackagesOf: class) includes: YPackage). 
			self assert: ((anOrganizer extendingPackagesOf: class theMetaClass) includes: YPackage). 
			class theMetaClass removeSelector: #stubMethod2.
			"there should be no differences made between class and metaClass:"
			self deny: ((anOrganizer extendingPackagesOf: class) includes: YPackage). 
			self deny: ((anOrganizer extendingPackagesOf: class theMetaClass) includes: YPackage). 
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'cyrilledelaunay 5/27/2011 11:01'!
testRemoveAllExtensionMethodsFromMetaClassUpdateOrganizerExtensionsDictionary
	"test that In a class (from the meta side), when we remove all the method extended by the same external package, the class get unregistered from the classExtendingPackagesMapping of the organizer "

	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class YPackage|
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage  := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class theMetaClass inCategory: '*yyyyy'. 
			self createMethodNamed: 'stubMethod2' inClass: class theMetaClass  inCategory: '*yyyyy'. 
			
			self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class theMetaClass). 
			self assert: (YPackage includesExtensionSelector: #stubMethod2 ofClass: class theMetaClass). 
			class theMetaClass removeSelector: #stubMethod.
			"there should be no differences made between class and metaClass:"
			self assert: ((anOrganizer extendingPackagesOf: class) includes: YPackage). 
			self assert: ((anOrganizer extendingPackagesOf: class theMetaClass) includes: YPackage). 
			class theMetaClass removeSelector: #stubMethod2.
			"there should be no differences made between class and metaClass:"
			self deny: ((anOrganizer extendingPackagesOf: class) includes: YPackage). 
			self deny: ((anOrganizer extendingPackagesOf: class theMetaClass) includes: YPackage). 
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'cyrilledelaunay 5/27/2011 10:32'!
testRemoveAllExtensionMethodsRemoveTheClassFromExtendedClasses
	"test that In a class, when we remove all the method extended by the same external package, the class get unregistered from the extended classes of the external package "

	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class YPackage|
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage  := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'. 
			self createMethodNamed: 'stubMethod2' inClass: class  inCategory: '*yyyyy'. 
			
			self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
			self assert: (YPackage includesExtensionSelector: #stubMethod2 ofClass: class). 
			class removeSelector: #stubMethod.
			self assert: (YPackage extendsClass: class). 
			class removeSelector: #stubMethod2.
			self deny: (YPackage extendsClass: class). 
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'cyrilledelaunay 5/27/2011 10:53'!
testRemoveAllExtensionMethodsRemoveTheMetaClassFromExtendedClasses
	"test that In a class (from the class side), when we remove all the method extended by the same external package, the class get unregistered from the extended classes of the external package "

	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class YPackage|
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage  := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class theMetaClass inCategory: '*yyyyy'. 
			self createMethodNamed: 'stubMethod2' inClass: class theMetaClass  inCategory: '*yyyyy'. 
			
			self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class theMetaClass ). 
			self deny: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
			self assert: (YPackage includesExtensionSelector: #stubMethod2 ofClass: class theMetaClass ).
			self deny: (YPackage includesExtensionSelector: #stubMethod2 ofClass: class ). 
			class theMetaClass removeSelector: #stubMethod.
			self assert: (YPackage extendsClass: class theMetaClass ). 
			self assert: (YPackage extendsClass: class ). 
			class theMetaClass removeSelector: #stubMethod2.
			self deny: (YPackage extendsClass: class).
			self deny: (YPackage extendsClass: class theMetaClass).
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'cyrilledelaunay 5/27/2011 10:57'!
testRemoveAllExtensionMethodsUpdateOrganizerExtensionsDictionary
	"test that In a class, when we remove all the method extended by the same external package, the class get unregistered from the classExtendingPackagesMapping of the organizer "

	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class YPackage|
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage  := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'. 
			self createMethodNamed: 'stubMethod2' inClass: class  inCategory: '*yyyyy'. 
			
			self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
			self assert: (YPackage includesExtensionSelector: #stubMethod2 ofClass: class). 
			class removeSelector: #stubMethod.
			self assert: ((anOrganizer extendingPackagesOf: class) includes: YPackage). 
			class removeSelector: #stubMethod2.
			self deny: ((anOrganizer extendingPackagesOf: class) includes: YPackage).
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on categories' stamp: 'StephaneDucasse 4/29/2011 21:31'!
testRemoveCategoryRemoveThePackage
	"test that when we remove a category, the corresponding RPackage is removed from the organizer"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage class|
			self addXCategory.
			self assert: (anOrganizer includesPackageNamed: #XXXXX).
			Smalltalk organization removeCategory: 'XXXXX'.
			self deny: (anOrganizer includesPackageNamed: #XXXXX).
			]
		] ensure: [ self toDoAtTheEnd ]
		
	! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on categories' stamp: 'StephaneDucasse 4/29/2011 21:32'!
testRemoveCategoryWhenMCPackageExistDoesNotRemoveThePackage
	"test that when we remove a category (usually from the class browser), and that a corresponding MCPackage exist, the RPackage still exist till the MCPackage has been unloaded"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage class mCPackage|
			"let's create the category and the MCPackage"
			self addXCategory.
			mCPackage := MCWorkingCopy forPackage: (MCPackage new name: 'XXXXX').
			"we make sure that the RPackage has been create (only one time)"
			self assert: (anOrganizer includesPackageNamed: #XXXXX).
			self assert: (anOrganizer packages select: [:each | each name = 'XXXXX']) size = 1.
			"we remove the category"
			Smalltalk organization removeCategory: 'XXXXX'.
			"we make sure the RPackage still exist"
			self assert: (anOrganizer includesPackageNamed: #XXXXX).
			
			"for now, this no longer the expected behavior. See comment in RPackageOrganizer>>update:"
			"mCPackage unregister.
			self deny: (anOrganizer includesPackageNamed: #XXXXX)."
			]
		] ensure: [ self toDoAtTheEnd ]
		
	! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 4/29/2011 21:33'!
testRemoveClassUnregisterTheClassDefinedMethodsFromItsPackage
	"test that when we remove a class, the class methods defined in the parent package (not extensions) are all removed  from its parent Package"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class|
			self addXCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: 'classic category'. 
			
			self assert: (XPackage includesDefinedSelector: #stubMethod ofClass: class). 
			Smalltalk removeClassNamed: 'NewClass'. 
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class). 
			self deny: (XPackage includesExtensionSelector: #stubMethod ofClass: class). 
			self deny: (XPackage includesSelector: #stubMethod ofClass: class). 
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 4/29/2011 21:54'!
testRemoveClassUnregisterTheClassExtensionMethodsFromTheCorrespondingPackage
	"test that when we remove a class, the class extension methods are all removed  from theire corresponding parent Package"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class|
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'. 
			
			self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
			Smalltalk removeClassNamed: 'NewClass'. 
			self deny: (YPackage includesDefinedSelector: #stubMethod ofClass: class). 
			self deny: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
			self deny: (YPackage includesSelector: #stubMethod ofClass: class). 
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 4/29/2011 21:34'!
testRemoveClassUnregisterTheClassFromItsPackage
	"test that when we remove a class, the class is removed from its parent Package"	

	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class|
			self addXCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self assert: (XPackage includesClass: class). 
			Smalltalk removeClassNamed: 'NewClass'. 
			self deny: (XPackage includesClass: class) ]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 5/8/2011 15:20'!
testRemoveClassUpdateTheorganizer
	"test that when we remove a class, the organizer is updated so that the class is no longer present in the  'classPackageDictionary' dictionary"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class|
			self addXCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self assert: (anOrganizer includesPackageBackPointerForClass: class). 
			Smalltalk removeClassNamed: 'NewClass'. 
			self deny: (anOrganizer includesPackageBackPointerForClass: class) ]
		] ensure: [ self toDoAtTheEnd. ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'cyrilledelaunay 5/9/2011 14:48'!
testRemoveExtensionMethodDoesNotRemoveExtendingPackage
	"test that when we remove a method that is an extension from an external package, the package is not removed from the the class extending packages if another extension still exist"

	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class YPackage|
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage  := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'. 
			self createMethodNamed: 'stubMethod2' inClass: class  inCategory: '*yyyyy'. 
			
			self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
			class removeSelector: #stubMethod.
			self assert: ((anOrganizer extendingPackagesOf: class) includes: YPackage).  
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'cyrilledelaunay 5/9/2011 14:50'!
testRemoveExtensionMethodDoesRemoveExtendingPackage
	"test that when we remove a method that is an extension from an external package, the package is  removed from the the class extending packages if no other extension method exist"

	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class YPackage|
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage  := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'. 
			
			self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
			class removeSelector: #stubMethod.
			self assert: ((anOrganizer extendingPackagesOf: class) isEmpty).  
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'StephaneDucasse 4/29/2011 21:54'!
testRemoveExtensionMethodRemoveMethodFromItsPackage
	"test that when we remove a method that is an extension from an external package, the method is removed from this package"

	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class YPackage|
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage  := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'. 
			
			self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
			class removeSelector: #stubMethod.
			self deny: (YPackage includesDefinedSelector: #stubMethod ofClass: class). 
			self deny: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
			self deny: (YPackage includesSelector: #stubMethod ofClass: class). 
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on MCPackages' stamp: 'cyrilledelaunay 5/4/2011 13:37'!
testRemoveMCPackageRemoveThePackage
	"test that when we remove a MC Package, the corresponding RPackage is removed from the organizer"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage class|
			MCWorkingCopy forPackage: (MCPackage new name: 'XXXXX').
			self assert: (anOrganizer includesPackageNamed: #XXXXX).
			(MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX']) unregister.
			"for now, this is no longer the expected behavior. See comment in RPackageOrganizer>>update:"
			"self deny: (anOrganizer includesPackageNamed: #XXXXX)."
			]
		] ensure: [ self toDoAtTheEnd ]
		
	! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'cyrilledelaunay 5/13/2011 14:51'!
testRemoveMethodComingFromTraitDoesNotRemoveMethodFromTraitPackage
	"test that when we remove a class method coming from a trait, the method is not removed from the parent package of the trait"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class trait|
			self addXCategory.
			self addYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			trait := self createNewTraitNamed: 'NewTrait' inCategory: 'YYYYY'.
			self createMethodNamed: 'stubMethod' inClass: trait  inCategory: 'classic protocol'. 
			class setTraitComposition: (trait) asTraitComposition.
			
			self assert: (YPackage includesDefinedSelector: #stubMethod ofClass: trait). 
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class). 
			class removeSelector: #stubMethod.
			self assert: (YPackage includesDefinedSelector: #stubMethod ofClass: trait). 
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class). 
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'StephaneDucasse 4/29/2011 21:36'!
testRemoveMethodRemoveMethodFromItsPackage
	"test that when we remove a method, the method is remod from the package in which the method is defined"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class|
			self addXCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: 'classic category'. 
			
			self assert: (XPackage includesDefinedSelector: #stubMethod ofClass: class). 
			class removeSelector: #stubMethod.
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class). 
			self deny: (XPackage includesExtensionSelector: #stubMethod ofClass: class). 
			self deny: (XPackage includesSelector: #stubMethod ofClass: class). 
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'cyrilledelaunay 5/13/2011 14:59'!
testRemoveTraitMethod
	"test that when we remove a method from a trait used by a class, the method is removed from the trait package normally"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class trait|
			self addXCategory.
			self addYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			trait := self createNewTraitNamed: 'NewTrait' inCategory: 'YYYYY'.
			self createMethodNamed: 'stubMethod' inClass: trait  inCategory: 'classic protocol'. 
			class setTraitComposition: (trait) asTraitComposition.
			
			self assert: (YPackage includesDefinedSelector: #stubMethod ofClass: trait). 
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class). 
			trait removeSelector: #stubMethod.
			self deny: (YPackage includesDefinedSelector: #stubMethod ofClass: trait). 
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class). 
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on categories' stamp: 'cyrilledelaunay 5/23/2011 11:23'!
testRenameCategoryAlsoRenameAllExtensionProtocols
	"test that when we rename a category, all corresponding extension protocols in the system are renamed"
	"self debug: #testRenameCategoryAlsoRenameAllExtensionProtocols"
	
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer  |
			|XPackage YPackage ZPackage classInY classInZ| 
			self addXCategory.
			self addYCategory.
			self addZCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			ZPackage := anOrganizer packageNamed: #ZZZZZ.
			
			classInY := self createNewClassNamed: 'ClassInYPackage' inCategory: 'YYYYY'.
			classInZ := self createNewClassNamed: 'ClassInZPackage' inCategory: 'ZZZZZ'. 
			
			self createMethodNamed:  'extensionFromXInClassInY'  inClass: classInY   inCategory: '*XXXXX'. 
			self createMethodNamed:  'longNameExtensionFromXInClassInY'  inClass: classInY   inCategory: '*XXXXX-subcategory'.     
			self createMethodNamed:  'extensionFromXInClassInZ'  inClass: classInZ   inCategory: '*XXXXX'.     
			    
			Smalltalk organization renameCategory: 'XXXXX' toBe: 'NewCategoryName'.
			
			self assert: (XPackage name = 'NewCategoryName').
			self assert: (classInY >> #extensionFromXInClassInY) category = '*NewCategoryName'.
			self assert: (classInY >> #longNameExtensionFromXInClassInY) category = '*NewCategoryName'.
			self assert: (classInZ >> #extensionFromXInClassInZ) category = '*NewCategoryName'.
		]
	] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on categories' stamp: 'StephaneDucasse 4/25/2011 11:58'!
testRenameCategoryChangeTheNameOfThePackage
	"test that when we rename a category, the RPackage corresponding is updated with this new name"
	
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer |
			|XPackage| 
			self addXCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			
			Smalltalk organization renameCategory: 'XXXXX' toBe: 'YYYYY'.
			self assert: (XPackage name = 'YYYYY').
		]
	] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on categories' stamp: 'StephaneDucasse 4/29/2011 21:37'!
testRenameCategoryUpdateTheOrganizer
	"test that when we rename a category, the organizer dictionary is update with this new name, so that we can access the package with this new name as key"

	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer |
			|XPackage| 
			self addXCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			Smalltalk organization renameCategory: 'XXXXX' toBe: 'YYYYY'.
			self assert: (anOrganizer packageNamed: 'YYYYY' asSymbol) = XPackage.
			self deny: (anOrganizer includesPackageNamed: #XXXXX).
		]
	] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 4/29/2011 21:37'!
testRenameClassUpdateClassDefinedInTheParentPackage
	"test that when we rename a class, the classes Defined in the parent package includes the new name"
	"self debug: #testRenameClassUpdateClassDefinedInTheParentPackage"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class refactoring|
			self addXCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			
			class := self createNewClassNamed: 'RPackageOldStubClass' inCategory: 'XXXXX'.	
			class rename: 'RPackageNewStubClass'.
			
			self assert: (XPackage includesClassNamed: 'RPackageNewStubClass'). 
			self deny: (XPackage includesClassNamed: 'RPackageOldStubClass'). 
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 4/29/2011 21:37'!
testRenameClassUpdateClassDefinedSelectorsInTheParentPackage
	"test that when we rename a class, the  'classDefinedSelectors' dictionary of the parent package is updated with the new name. There fore we test that we can correctly access the selector from the package by specifying the right name (the new name)"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class refactoring|
			self addXCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'RPackageOldStubClass' asSymbol inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: 'classic category'.
			
			class rename: 'RPackageNewStubClass'.
			
			self assert: (XPackage includesDefinedSelector: #stubMethod ofClassName: 'RPackageNewStubClass' asSymbol ). 
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClassName: 'RPackageOldStubClass' asSymbol ).
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 4/29/2011 21:53'!
testRenameClassUpdateClassExtensionSelectorsInTheExtendingPackages
	"test that when we rename a class, the  'classExtensionSelectors' dictionaries of the extending packages  are updated with the new name. Therfore we test that we can correctly access the selectors from the package by specifying the right name (the new name)"

	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class refactoring|
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'RPackageOldStubClass' asSymbol inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'.
			
			class rename: 'RPackageNewStubClass'.
			
			self assert: (YPackage includesExtensionSelector: #stubMethod ofClassName: 'RPackageNewStubClass' asSymbol ). 
			self deny: (YPackage includesExtensionSelector: #stubMethod ofClassName: 'RPackageOldStubClass' asSymbol ).
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 4/29/2011 21:53'!
testRenameClassUpdateMetaclassDefinedSelectorsInTheParentPackage
	"test that when we rename a class, the  'metaclassDefinedSelectors' dictionary of the parent package is updated with the new name. Ther fore we test that we can correctly access the selector from the package by specifying the right name (the new name)"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage  class refactoring|
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'RPackageOldStubClass' asSymbol inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClassSideOfClass: class  inCategory: 'classic category'.
			
			class rename: 'RPackageNewStubClass'.
			
			self assert: (XPackage  includesDefinedSelector: #stubMethod ofMetaclassName: 'RPackageNewStubClass' asSymbol ). 
			self deny: (XPackage  includesDefinedSelector: #stubMethod ofMetaclassName: 'RPackageOldStubClass' asSymbol ).
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 4/29/2011 21:52'!
testRenameClassUpdateMetaclassExtensionSelectorsInTheExtendingPackages
	"test that when we rename a class, the  'metaclassExtensionSelectors' dictionaries of the extending packages  are updated with the new name. Ther fore we test that we can correctly access the selectors from the package by specifying the right name (the new name)"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class refactoring|
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'RPackageOldStubClass' asSymbol inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClassSideOfClass: class  inCategory: '*yyyyy'.
			
			class rename: 'RPackageNewStubClass'.
			
			self assert: (YPackage includesExtensionSelector: #stubMethod ofMetaclassName: 'RPackageNewStubClass' asSymbol ). 
			self deny: (YPackage includesExtensionSelector: #stubMethod ofMetaclassName: 'RPackageOldStubClass' asSymbol ).
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 4/29/2011 21:52'!
testRenameClassUpdateOrganizerClassExtendingPackagesMapping
	"test that when we rename a class, the classExtendingPackages dictionary of the organizer is updated with the new name, so that we can access the packages when specifying the new name"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage YPackage class refactoring|
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'RPackageOldStubClass' inCategory: 'XXXXX'.
			self createMethodNamed: #stubMethod inClass: class inCategory: '*yyyyy'. 
			
			class rename: 'RPackageNewStubClass'.
			
			self assert: ((anOrganizer extendingPackagesOfClassNamed: 'RPackageNewStubClass' asSymbol) includes: YPackage). 
			self deny: ((anOrganizer extendingPackagesOfClassNamed: 'RPackageOldStubClass' asSymbol) includes: YPackage). 
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'StephaneDucasse 4/29/2011 21:52'!
testRenameClassUpdateOrganizerClassPackageMapping
	"test that when we rename a class, the classPackage dictionary of the organizer is updated with the new name, so that we can access the package when specifying the new name"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			|XPackage class refactoring|
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'RPackageOldStubClass' inCategory: 'XXXXX'.
			
			class rename: 'RPackageNewStubClass'.
			
			self assert: ((anOrganizer packageOfClassNamed: 'RPackageNewStubClass' asSymbol) = XPackage). 
			self assert: ((anOrganizer packageOfClassNamed: 'RPackageOldStubClass' asSymbol) = nil). 
			]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on categories' stamp: 'cyrilledelaunay 5/27/2011 12:31'!
testRenameUnknownCategoryCreatesNewRPackage
	"test that when we rename a category that is not registered in RPackage , it does not raise errors and simply create a new package. We need this behaviour as for now, create a new category with the class browser does not emit the corrects events, and therefore RPackage can not be directly updated"
	
	[RPackage 
		withOrganizer: self emptyOrganizer  
		do: [ :anOrganizer |
			SystemChangeNotifier uniqueInstance doSilently: [
				self addXCategory.
				].
			self deny: (anOrganizer includesPackageNamed: #XXXXX).
			Smalltalk organization renameCategory: 'XXXXX' toBe: 'YYYYY'.
			self assert: (anOrganizer includesPackageNamed: #YYYYY).
			
			]
		] ensure: [ self toDoAtTheEnd.]
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on protocols' stamp: 'StephaneDucasse 4/24/2011 23:30'!
testReorganizeClassByAddingNewProtocolDoesNothing
	"test that when we reoganized a class by adding a category, nothing change from  RPackage point of vue."
	
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer |
			|XPackage YPackage class| 
			self addXCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'newMethod' inClass: class inCategory: 'xxxxx'.  
			class organization addCategory: 'accessing' before: nil.
			
			self assert: (XPackage includesClass: class).
			self assert: (XPackage includesDefinedSelector: #newMethod ofClass: class).
			self assert: (XPackage extensionMethods size = 0).
		]
	] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on protocols' stamp: 'cyrilledelaunay 5/4/2011 16:13'!
testReorganizeClassByRemovingClassicCategoryRemoveMethodsFromTheParentPackage
	"test that when we reoganized a class by removing a classic category (a category not beginning with '*'), the methods from this category are removed from the parent RPackage of the class"
	
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer |
			|XPackage YPackage class| 
			self addXCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: 'classic category'.
			class removeCategory: 'classic category'.
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class).
		]
	] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on protocols' stamp: 'StephaneDucasse 4/29/2011 21:52'!
testReorganizeClassByRemovingExtensionCategoryRemoveMethodsFromTheExtendingPackage
	"test that when we reoganized a class by removing an extension category (a category beginning with '*'), all the methods that were inside are removed from the extending package"
	 
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer |
			|XPackage YPackage class| 
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage  := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'.
			
			class removeCategory: '*yyyyy'.
			
			self deny: (YPackage includesExtensionSelector: #stubMethod ofClass: class).
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class).
		]
	] ensure: [ self toDoAtTheEnd.]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on protocols' stamp: 'StephaneDucasse 4/29/2011 21:52'!
testReorganizeClassByRenamingClassicCategoryToExtensionCategoryMoveAllMethodsFromParentPackageToExtendingPackage
	"self debug:#testReoganizeClassByRenamingClassicCategoryToExtensionCategoryMoveAllMethodsFromParentPackageToExtendingPackage"
	"test that when we reoganized a class by renaming a  classic category (a category not beginning with '*') to an extension category, all the methods are moved from the  parent package of the class to the extending package"
	
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer |
			|XPackage YPackage class| 
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage  := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: 'classic category'.
			self createMethodNamed: 'stubMethod2' inClass: class  inCategory: 'classic category'.
			
			class organization renameCategory: 'classic category' toBe: '*yyyyy'.
			
			self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class).
			self assert: (YPackage includesExtensionSelector: #stubMethod2 ofClass: class).
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class).
			self deny: (XPackage includesDefinedSelector: #stubMethod2 ofClass: class).
		]
	] ensure: [
			self toDoAtTheEnd.
		"|mCPackage|
			Smalltalk removeClassNamed: 'NewClass'.
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].
			mCPackage := (MCWorkingCopy allManagers detect: [:each | each packageName = 'Yyyyy'] ifNone: [nil]) .
			mCPackage ifNotNil: [mCPackage unregister].	
			Smalltalk organization removeCategory: 'XXXXX'.
			Smalltalk organization removeCategory: 'YYYYY'."
		]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on protocols' stamp: 'StephaneDucasse 4/29/2011 21:51'!
testReorganizeClassByRenamingClassicCategoryToExtensionCategoryMoveMethodsFromParentPackageToExtendingPackage
	"test that when we reoganized a class by renaming a  classic category (a category not beginning with '*') to an extension category, all the methods are moved from the  parent package of the class to the extending package"
	
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer |
			|XPackage YPackage class| 
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage  := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: 'classic category'.
			class organization renameCategory: 'classic category' toBe: '*yyyyy'.
			self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class).
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class) ]
	] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on protocols' stamp: 'StephaneDucasse 4/29/2011 21:51'!
testReorganizeClassByRenamingExtensionCategoryToAnotherExtensionCategoryAddMethodsToTheNewPackage
	"test that when we reoganized a class by renaming an extension category (a category beginning with '*') to another extension category, all the methods are moved to the new extendingPackage"
	
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer |
			|XPackage YPackage ZPackage class| 
			self addXYZCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage  := anOrganizer packageNamed: #YYYYY.
			ZPackage  := anOrganizer packageNamed: #ZZZZZ.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'.
			class organization renameCategory: '*yyyyy' toBe: '*zzzzz'.
			self assert: (ZPackage includesExtensionSelector: #stubMethod ofClass: class).
			self deny: (YPackage includesExtensionSelector: #stubMethod ofClass: class).
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class) ]
		] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on protocols' stamp: 'StephaneDucasse 4/29/2011 21:51'!
testReorganizeClassByRenamingExtensionCategoryToClassicCategoryMoveAllMethodsFromExtendingPackageToParentPackage
	"test that when we reoganized a class by renaming an extension category (a category beginning with '*') to a classic category, all the methods are moved from the  extendingPackage to the parent package of the class"
	"self debug: #testReoganizeClassByRenamingExtensionCategoryToClassicCategoryMoveAllMethodsFromExtendingPackageToParentPackage"
	
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer |
			|XPackage YPackage class| 
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage  := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'.
			self createMethodNamed: 'stubMethod2' inClass: class  inCategory: '*yyyyy'.
			
			class organization renameCategory: '*yyyyy' toBe: 'classic category'.
			
			self deny: (YPackage includesExtensionSelector: #stubMethod ofClass: class).
			self deny: (YPackage includesExtensionSelector: #stubMethod2 ofClass: class).
			self assert: (XPackage includesDefinedSelector: #stubMethod ofClass: class).
			self assert: (XPackage includesDefinedSelector: #stubMethod2 ofClass: class).
		]
	] ensure: [ self toDoAtTheEnd ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on protocols' stamp: 'StephaneDucasse 4/29/2011 21:51'!
testReorganizeClassByRenamingExtensionCategoryToClassicCategoryMoveMethodsFromExtendingPackageToParentPackage
	"test that when we reoganized a class by renaming an extension category (a category beginning with '*') to a classic category, all the methods are moved from the  extendingPackage to the parent package of the class"
	
	[RPackage 
		withOrganizer: self emptyOrganizer
		do: [ :anOrganizer |
			|XPackage YPackage class| 
			self addXYCategory.
			XPackage := anOrganizer packageNamed: #XXXXX.
			YPackage  := anOrganizer packageNamed: #YYYYY.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'.
			class organization renameCategory: '*yyyyy' toBe: 'classic category'.
			self deny: (YPackage includesExtensionSelector: #stubMethod ofClass: class).
			self assert: (XPackage includesDefinedSelector: #stubMethod ofClass: class) ]
		] ensure: [ self toDoAtTheEnd. ]! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'StephaneDucasse 4/29/2011 21:50'!
testisDefinedAsPackageOrSubPackageInMC
	"test whether a given category is defined as a MC package or whether it is defined as top part of package"
	
	"self debug: #testisDefinedAsSubPackageInMC"
	"self debug: #testisDefinedAsPackageOrSubPackageInMC"
	
	[self assert: (RPackageOrganizer default isDefinedAsPackageOrSubPackageInMC: #'Compression').
	self deny: (RPackageOrganizer default isDefinedAsPackageOrSubPackageInMC: #'Compression-Archives').
	"Indeed compression-archives is a category but not a MC package"
	
	self assert: (RPackageOrganizer default isDefinedAsPackageOrSubPackageInMC: #'Collections').
	"#Collection is not defined as a package but as a package #'Collections-Unordered'"
	self assert: (RPackageOrganizer default isDefinedAsPackageOrSubPackageInMC: #'Collections-Unordered').
	self deny: (RPackageOrganizer default isDefinedAsPackageOrSubPackageInMC: #'Collections-Unordered-Test').
		] ensure: [ self toDoAtTheEnd ]
	! !

!RPackageMCSynchronisationTest methodsFor: 'setup' stamp: 'cyrilledelaunay 1/31/2011 17:59'!
toDoAtTheEnd

	MCPackageManager registerForNotifications.
	MCWorkingCopy registerForNotifications.! !
